<?php
namespace Teknisa\CRM\Controller;

use Teknisa\CRM\Controller\TeknisaCrud;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;

class crm20000CasoController extends TeknisaCrud
{
    protected $dataSourceName = 'crm_selectTeladeCaso_perf1';
    protected $parameterBagColumns = array("P_NRORG");

    protected $entityManager;
    protected $connection;
    protected $newCode;
    protected $environment;

    public function __construct ($crudService, $parameterBag, $entityManager, $newCode, $environment)
    {
        $this->entityManager = $entityManager;
        $this->newCode       = $newCode;
        $this->connection    = $entityManager->getConnection();
        $this->environment   = $environment;

        parent::__construct($crudService, $parameterBag);
    }

    public function save(DTO\Request $request, DTO\Response $response)
    {
        foreach($request->getDataSet()->getRows() as $row)
        {
            if (!array_key_exists('DTULTATU', $row)) $row['DTULTATU'] = null;
            if (!array_key_exists('IDATIVO', $row)) $row['IDATIVO'] = 'S';
            $row["CDFILIAL"] = $this->environment->getCdFilial();

            if ($row['__is_new'] && (!isset($row['NRSEQCONTATO'])))
            {   
               // if(!isset($row["CDCONTATO"])){
                    $row['NRSEQCONTATO'] = $this->newCode->getCodeByTableName("NRSEQCONTATO", $row["NRORG"]);
                //}else{
                    //$row['NRSEQCONTATO'] = !$row["CDCONTATO"];
                //}
                
                $row['NRCONTATO'] = $row['NRSEQCONTATO'];
                
                date_default_timezone_set('America/Sao_Paulo');
                $row['DTINCLUSAO'] = date('d/m/y');

                $response->addDataSet(new DataSet("DADOS", array("NRSEQCONTATO" => $row["NRSEQCONTATO"], "NRCONTATO" => $row["NRSEQCONTATO"], "DTINCLUSAO" => $row["DTINCLUSAO"])));
            }
            else
            {
                date_default_timezone_set('America/Sao_Paulo');
                $row['DTULTATU'] = date('d/m/y');
                $response->addDataSet(new DataSet("DADOS", array("DTULTATU" => $row["DTULTATU"])));
            }
        }

        parent::save($request, $response);
    }

    public function getSituacao(Row $row, Response $response)
    {
        $row = $row->getRow();

        $select = "SELECT DSSITUACAO AS NMSITUACAO
                     FROM CRM_SITUACAO
                    WHERE NRSEQSITUACAO = :NRSEQSITUACAO
                      AND NRORG         = :NRORG";

        $params = array
        (
            "NRSEQSITUACAO" => $row["NRSEQSITUACAO"],
            "NRORG"         => $row["NRORG"]
        );

        try
        {
            $std = $this->entityManager->getConnection()->prepare($select);
            $std->execute($params);
            $result = $std->fetchAll();
            $response->addDataSet(new DataSet("DADOS", array("NMSITUACAO" => $result[0]["NMSITUACAO"])));
        }
        catch(\Exception $e)
        {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        };
    }
    
    
    public function updateMotivo(Row $row){
        $row = $row->getRow();

        $update = "UPDATE CONTATO
                    SET CDMOTICONT   = :CDMOTICONT
                   WHERE NRSEQCONTATO = :NRSEQCONTATO AND NRORG = :NRORG";

        $params = array
        (
            "CDMOTICONT" => $row["CDMOTICONT"],
            "NRSEQCONTATO" => $row["NRSEQCONTATO"],
            "NRORG"         => $row["NRORG"]
        );

        $std = $this->entityManager->getConnection()->prepare($update);
        $std->execute($params);
 
    }
    
    public function insertEvolutionNextTask(Row $row, Response $response){
        
        $row = $row->getRow();
        
        $insertTask = "INSERT INTO FILACONTATO (DSFILACONTATO,DTPLANFILACONTA,CDOPERREAL,NRSEQFILACONTA,NRORG,DTINCLUSAO,NRORGINCLUSAO,
                               CDOPERINCLUSAO,NRPARCNEGOCIO,IDORIGFILACONTA,IDATIVO,NRCONTATO,NRSEQCONTATO, NRSEQCLASSIFICATAREFA)
                    VALUES (:DSFILACONTATO,
                            :DTPLANFILACONTA,
                            :CDOPERREAL,
                            :NRSEQFILACONTA,
                            :NRORG,
                             SYSDATE,
                            :NRORGINCLUSAO,
                            :CDOPERINCLUSAO,
                            :NRPARCNEGOCIO,
                             'M',
                             'S',
                            :NRCONTATO,
                            :NRSEQCONTATO,
                            :NRSEQCLASSIFICATAREFA)";
                            
        $row["NRSEQFILACONTA"] = $this->newCode->getCodeByTableName("FILACONTATO", $row["NRORG"]);
        
        $paramsInsertTask = array(
            "NRSEQFILACONTA"            => $row["NRSEQFILACONTA"],
            "DSFILACONTATO"             => $row["DSFILACONTATO"],
            "DTPLANFILACONTA"           => $row["DTPLANFILACONTA"],
            "CDOPERREAL"                => $row["CDOPERREAL"],
            "NRORG"                     => $row["NRORG"],
            "NRORGINCLUSAO"             => $row["NRORG"],
            "CDOPERINCLUSAO"            => $row["CDOPERREAL"],
            "NRPARCNEGOCIO"             => $row["NRPARCNEGOCIO"],
            "NRCONTATO"                 => $row["NRSEQCONTATO"],
            "NRSEQCONTATO"              => $row["NRSEQCONTATO"],
            "NRSEQCLASSIFICATAREFA"     => "2"
        );
          
        try
        {
            $std = $this->entityManager->getConnection()->prepare($insertTask);
            $std->execute($paramsInsertTask);
        }
        catch(\Exception $e)
        {
            throw $e;
        };
        
        $response->addDataSet(new DataSet("DADOS", array("NRSEQFILACONTA" => $row["NRSEQFILACONTA"])));
        
    }
    
    public function deactivateCaseTask(Row $row){
        
        $row = $row->getRow();
        
        $updateTask = "UPDATE FILACONTATO
                       SET IDATIVO = 'N'
                       WHERE NRSEQFILACONTA = :NRSEQFILACONTA";
                       
        $paramsUpdateTask = array(
            
            "NRSEQFILACONTA"    => $row["NRSEQFILACONTA"] 
            
        );
        
        try
        {
            $std = $this->entityManager->getConnection()->prepare($updateTask);
            $std->execute($paramsUpdateTask);
        }
        catch(\Exception $e)
        {
            throw $e;
        };
        
    }
    
    public function setTaskEndDate(Row $row){
        
        $row = $row->getRow();
        
        $updateTask = "UPDATE FILACONTATO
                       SET DTREALFILACONTA = SYSDATE
                       WHERE NRSEQFILACONTA = :NRSEQFILACONTA";
                       
        $paramsUpdateTask = array(
            
            "NRSEQFILACONTA"    => $row["NRSEQFILACONTA"]
            
        );
        
        try
        {
            $std = $this->entityManager->getConnection()->prepare($updateTask);
            $std->execute($paramsUpdateTask);
        }
        catch(\Exception $e)
        {
            throw $e;
        };
        
    }
    
    public function validadeCaseCustomCode(Row $row, Response $response){
        
        $row = $row->getRow();
        
        if($row["__is_new"]){
            
            $selectContactCode = "SELECT CDCASO
                        FROM CONTATO
                        WHERE UPPER(CDCASO) = '".strtoupper($row["CDCASO"])."'";
                        
            $paramsSelectContactCode = "";
                
        }else{
            
            $selectContactCode = "SELECT CDCASO
                        FROM CONTATO
                        WHERE UPPER(CDCASO) = '".strtoupper($row["CDCASO"])."'
                        AND NRSEQCONTATO <> :NRSEQCONTATO";
                        
            $paramsSelectContactCode = array(
                
                "NRSEQCONTATO"      =>      $row["NRSEQCONTATO"]
                    
            );
                
        }
                       
        try
        {
            $std = $this->entityManager->getConnection()->prepare($selectContactCode);
            $std->execute($paramsSelectContactCode);
            $result = $std->fetchAll();
            
            $response->addDataSet(new DataSet("DADOS", $result));
        }
        catch(\Exception $e)
        {
            throw $e;
        };
        
    }

}