function EvolucaoController(ScreenService, templateManager, Utilities, wizardEngine, ZHPromise, Operador_Repository, General, CRM_Utilities, restEngine, ChamadoController, evolutionTabController, ZHPromise) {

    /**
     * Função para mudar o readonly e o validations
     * dos campos de próxima tarefa baseado no resultado
     * da evolução.
    */
    this.checkNewTask = function(widget) {
        var currentRow = widget.currentRow;
        var fieldsNextTask = widget.fields.map(function(field) {
            if(field.fieldGroup && field.fieldGroup == "FG_AGENDAR_TAREFA")
                return field;
        }).filter(function(x) {
            return typeof x !== 'undefined';
        });
        if(currentRow.NEWTASK && currentRow.NEWTASK == "S") {
            fieldsNextTask.forEach(function(field) {
                field.readOnly = false;
                field.validations = {"required":true};
            });
        }else{
            fieldsNextTask.forEach(function(field) {
                field.readOnly = true;
                field.validations = {};
                field.applyDefaultValue();
            });    
        }
    };
    
    /**
     * Função para abrir a widget de seleção
     * de arquivos.
    */
    this.openAttachSwipe = function() {
        
        var fileUploadWidget = templateManager.container.getWidget('WDGT_SELECAO_ARQUIVOS');
        
        fileUploadWidget.isVisible = true;
        ScreenService.openSwipe(fileUploadWidget);
            
    };
    
    /**
     * Função para salvar os anexos selecionados
     * ao caso em questão.
     * 
     * @param {object} widget widget onde é
     * feito a seleção dos anexos.
    */
    this.uploadEvolutionFiles = function(widget, fromChamado) {
        
        var files = widget.currentRow.DSCAMINHO;
        
        var rows = [];
        
        files.forEach(function(file, fileIndex) {
            
            if(fromChamado)
                rows.push(this.setValuesForFileUpload(widget, fileIndex, fromChamado));
            else
                rows.push(this.setValuesForFileUpload(widget, fileIndex));
            
        }.bind(this));
        
        General.callRestEngine("product#/crm_anexocontato/save", "DataSet", rows).then(function() {
                
                this.closeAttachSwipe(widget);
                if(fromChamado) {
                    //Volta is new pra false
                    widget.currentRow.__is_new = false;
                    templateManager.container.getWidget('WDGT_ANEXOS_CHAMADO').reload();
                }else{
                    widget.parent.parent.reload();
                }
        }.bind(this)).catch(function() {
            widget.currentRow.__is_new = false;
        });
    };
    
    /**
     * Função para retornar uma row configurada
     * com as informações do anexo referente
     * ao ID, recebido como parâmetro, no array
     * de anexos na currentRow.
     * 
     * @param {object} widget widget onde foi selecionado
     * os anexos.Array
     * 
     * @param {int} fileIndex index do anexo no array de 
     * anexos presente na currentRow.
     * 
     * @return {object} currentRow row configurada com as informações
     * do anexo referente ao ID recebido.
    */
    this.setValuesForFileUpload = function(widget, fileIndex, fromChamado) {
        
        var currentRow = angular.copy(widget.currentRow);
        var loggedOperatorInfos = CRM_Utilities.getLoggedOperatorInfos();
        
        if(!fromChamado)
            var evolutionWidget = widget.parent.parent;
        else
            var chamadoWidget   = templateManager.container.getWidget(fromChamado);
        
        if(evolutionWidget) {
            currentRow.NRSEQCONTATO = evolutionWidget.currentRow.NRSEQCONTATO;
            currentRow.IDORIGEM = "EVOLUCAO";
            currentRow.NRORIGEM = evolutionWidget.currentRow.NRSEQEVOLCONT;
        }else if(chamadoWidget) {
            currentRow.NRSEQCONTATO = chamadoWidget.currentRow.NRSEQCONTATO;
            currentRow.IDORIGEM = "CONTATO";
            currentRow.NRORIGEM = chamadoWidget.currentRow.NRSEQCONTATO;
        }
        
        currentRow.NMARQUIVO = this.getFileNameWithoutExtension(currentRow.DSCAMINHO[fileIndex].name);
        currentRow.DSTIPO = this.getFileExtension(currentRow.DSCAMINHO[fileIndex].name);
        currentRow.VRTAMANHO = currentRow.DSCAMINHO[fileIndex].size;
        currentRow.IDATIVO = "S";
        currentRow.NRORG = loggedOperatorInfos.NRORG;
        currentRow.CDOPERADOR = loggedOperatorInfos.CDOPERADOR;
        
        return currentRow;
        
    };
    
    /**
     * Função para retornar a extenção de um arquivo
     * extraida do seu nome.
     * 
     * @param {string} fileName nome completo do arquivo.
    */
    this.getFileExtension = function(fileName) {
        
        return fileName.replace(/^(.*)\.(.*)$/, "$2");
        
    };
    
    /**
     * Função para retornar o nome do arquivo sem
     * sua extenção.
     * 
     * @param {string} fileName nome completo do arquivo.
    */
    this.getFileNameWithoutExtension = function(fileName) {
        
        return fileName.replace(/^(.*)\.(.*)$/, "$1");
            
    };
    
    /**
     * Função para fechar o swipe de upload de imagens.
     * 
     * @param {object} widget widget de upload de imagens.
    */
    this.closeAttachSwipe = function(widget) {
        ScreenService.closeSwipe().then(function() {
            widget.isVisible = false;
            widget.parent.parent.list();
            widget.parent.parent.activate();
        });
    };
    
    /**
     * Função para fazer o download de uma arquivo
     * salvo no servidor.
     * 
     * @param {object} widget widget que contém
     * as informações do arquivo a ser baixado 
     * em sua currentRow.
    */    
    this.downloadFile = function(widget) {
        
        window.open(window.fileServerBaseUrl + widget.currentRow.DSCAMINHO);
            
    };
    
    /**
     * Verifica se já existem tarefas relacionadas
     * aquele parceiro
     * 
     * @param {object} widget widget de evolução
    */
    this.verificaTarefasAbertas = function(currentRow) {
        currentRow.CRM_NRORGTRAB = getLocalVar('CRM_NRORGTRABALHO');
        
        var zhPromise = ZHPromise.defer();
        var paramsRestEngine = {
            "requestType": "Row",
            "row": currentRow
        };
        
        restEngine.post('/verificaTarefas', paramsRestEngine).
            then(function(response) {
                if(response.dataset.DADOS.TAREFAS) {
                    var tarefas = response.dataset.DADOS.TAREFAS;
                    var operadores = '';
                    tarefas.forEach(function(tarefa) {
                        operadores +='<br>'+tarefa;
                    });
                    ScreenService.confirmMessage('Este parceiro possui tarefas "Em Aberto" para o(s) seguinte(s) operador(es) <br>'+operadores+'<br><br> Deseja continuar? ', 'CONFIRMATION',
                    function(success) {
                        zhPromise.resolve();
                    }.bind(this),
                    function(fail) {
                        zhPromise.reject();
                    });
                }else{
                    zhPromise.resolve();
                }
            }.bind(this));
        return zhPromise.promise;
    };
    
     /**
     * Preapara widgets de evolução, verificando tarefas
     * abertas e chama o método de save
     * 
     * @param {object} widget widget de evolução
    */
    this.checkEvolution = function(widget) {
        //prepara widget para a verificação
        if(!widget.currentRow.NRPARCNEGOCIO)
            widget.currentRow.NRPARCNEGOCIO = widget.parent.parent.currentRow.NRPARCNEGOCIO;
        
        if(widget.isValid()) {
            this.verificaTarefasAbertas(widget.currentRow).
                then(function() {
                    if(!widget.currentRow.EVOLUCAOPRECONCLUSAO === true) {
                        this.saveEvolution(widget);
                    }else{
                        ScreenService.closeSwipe();
                        templateManager.container.WIDGETEVOLUCAO = widget;
                        ChamadoController.concluirChamado(widget.parent);
                    }
                    widget.activate();
                }.bind(this)).
                catch(function() {
                    General.widgetCancelForm(widget);
                }.bind(this));
        }    
    };
    
     /**
     * Salva widgets de evolução, gerando nova tarefa
     * baseado no resultado da evolução
     * 
     * @param {object} widget widget de evolução
    */
    this.saveEvolution = function(widget) {
        var currentRow = widget.currentRow;
        currentRow.CRM_NRORGTRAB = getLocalVar('CRM_NRORGTRABALHO');
        widget.fields.forEach(function(field) {
            if(!Util.isEmptyOrBlank(field.fieldGroup)) {
                if(field.fieldGroup == 'FG_AGENDAR_TAREFA') {
            	    field.readOnly = true;
                }
            }
        });
        General.widgetSave(widget).then(function(responseEvol) {
            if (responseEvol.dataset.DADOS) {
                widget.currentRow.NRSEQEVOLCONT = responseEvol.dataset.DADOS.NRSEQEVOLCONT;
            }
            
            this.checkMail(widget).then(function() { 
                if(currentRow.NEWTASK && currentRow.NEWTASK == "S") {
                    var paramsRestEngine = {
                        "requestType": "Row",
                        "row": currentRow 
                    };
                    
                    restEngine.post('/generalInsertTask', paramsRestEngine). 
                        then(function(responseTask) {
                            ScreenService.showMessage('Nova Tarefa Criada!');    
                        }.bind(this)).
                        catch(function(fail) {
                            ScreenService.showMessage('Falha ao criar Tarefa.');
                        }.bind(this));
                        
                }
                if(widget.afterSave) {
                    widget.afterSave.forEach(function(method) {
                        eval(method);    
                    });
                }
                widget.reload();
            }.bind(this));
        }.bind(this));
    };
    
    this.checkMail = function(widget) {
        var promise = ZHPromise.defer();
        if(!Util.isEmptyOrBlank(widget.currentRow.NRSEQINTERLOCUTOR) && widget.currentRow.IDENVIAEMAIL === 'S') {
                    ScreenService.confirmMessage('Deseja enviar um email para '+widget.currentRow.NMINTERLOCUTOR+'?', 'QUESTION',
            function(yes) {
                evolutionTabController.openWidgetSendMail(widget).then(function() {
                    promise.resolve();
                });
            }, function(no) {
                promise.resolve();
            });
        }else{
            promise.resolve();
        }
        return promise.promise;
    };
    
    this.saveChamadoExterno = function(widget) {
        General.widgetSave(widget).then(function(response) {
            if(response.dataset.DADOS && response.dataset.DADOS.NRSEQCONTATO)
                widget.currentRow.NRSEQCONTATO = parseInt(response.dataset.DADOS.NRSEQCONTATO);
                widget.reload().then(function() {
                    //seta is new pra true pra fazer a inserção do anexo
                    if(widget.currentRow.DSCAMINHO) {
                        widget.currentRow.__is_new = true;
                        this.uploadEvolutionFiles(widget, 'WDGT_ABERTURA_CHAMADO');
                    }
                }.bind(this));
        }.bind(this));
    }
    
    this.setDTFilter = function(args) {
        var wid = args.owner;
        var filter = wid.dataSourceFilter;
        
        filter.forEach(function(f) {
            if(f.name === 'DTHREVOLCONT' || f.name === 'DATAINCLUSAO' || f.name === 'DTREALFILACONTA' || f.name === 'DTPREVFECHA') {
                f.value[1] = f.value[1] + ' 23:59';
            }
        });
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('EvolucaoController', EvolucaoController);
});