function CRM_Utilities(ScreenService, templateManager, Operador_Repository, ZHPromise, restEngine, General, ValidationEngine, ZhUtil, ZhFilterPreferences, metaDataFactory, Messages) {
    
    this.controllers = {};
    
    this.addController = function(controller, controllerName) {
        this.controllers[controllerName] = controller;
    };
    
    this.getController = function(controllerName) {
        var controller = this.controllers[controllerName];
        
        return controller;
    };
    
    /**
     * Função para limpar todos os valores de uma widget recebida
     * como parâmetro.
     * 
     * @param {obejct} widget widget na qual será removido os valores
     * de todas as fields.
     * 
    */
    this.cleanWidgetFields = function(widget){
        widget.fields.forEach(function(field){
            field.value("");    
        });   
    };
    
    /**
     * Função para retornar as informações do
     * operador logado.
     * 
     * @return {object} loggedOperatorInfos objeto
     * que contém todas as informações do operador
     * logado.
    */
    this.getLoggedOperatorInfos = function(){
        
        var loggedOperatorInfos = templateManager.LOGGEDOPERATOR;
        
        return loggedOperatorInfos;
            
    };
	
	/**
     * Função para setar as informações do operador 
     * logado em um array na widget recebida.
     * 
     * @param {object} destinationObject objeto destino 
     * das informações do operador logado.
     * 
    */
	this.setOperatorLoggedInfosInCurrentRow = function(destinationObject){
        
        return Operador_Repository.retornaOperadorLogado().then(function(operator){
        
            if(!operator[0]){
                
                ScreenService.showMessage("Não foi possível carregar os dados do operador logado. Entre em contato com um administrador sobre o erro.");
                return;
            }
        
            destinationObject.OPERATORLOGGED = operator[0];
            
            }, 
            function(error){
                ScreenService.showMessage("Não foi possível carregar os dados do operador logado. Entre em contato com um administrador sobre o erro.");
            }
        );  
    };
    
    /**
     * Retorna o index da widget localizada, pelo nome
     * recebido como parâmetro da função, no array de widgets,
     * também recebido como parâmetro.
     * 
     * @param {object} widgets array de widgets onde é feito
     * a busca.
     * 
     * @ param {string} widgetName nome da widget a ser
     * localizada.
     * 
     * @param {int} indexWidgetObject index da widget localizada
     * dentro do array de wigets.
    */
    this.getWidgetPosition = function(widgets, widgetName){
        
        var indexWidgetObject;
        
        widgets.forEach(function(widget, indexWidget){
            
            if(widget.name === widgetName){
                indexWidgetObject = indexWidget;
                return;
            }
                
        });
        
        return indexWidgetObject;
            
    };
    
    /**
     * Função para identificar o index de um objeto
     * dentro de um dataSourceFilter, de acordo com 
     * seu nome.
     * 
     * @param {object} dataSourceFilter objeto da busca.
     * 
     * @param {String} name propriedade no objeto pelo qual
     * é feito a busca.
     * 
     * @return {int} indexFilterObject index Do objeto 
     * encontrado.
     * 
    */
    this.getFilterPosition = function(dataSourceFilter, name){
        var indexFilterObject;
        
        dataSourceFilter.forEach(function(value,i){
            if(value.name === name){
                indexFilterObject = i;
                return;
            }    
        });
        return indexFilterObject;
    };
    
    /**
     * Função para identificar o index de um objeto
     * dentro de um FieldGroups, de acordo com 
     * seu nome.
     * 
     * @param {object} fieldGroup objeto da busca.
     * 
     * @param {String} name propriedade no objeto 
     * pelo qual é feito a busca.
     * 
     * @return {int} indexFieldGroupObject index Do objeto 
     * encontrado.
    */
    this.getFieldGroupPosition = function(fieldGroups, name){
      
        var indexFieldGroupObject;
      
        fieldGroups.forEach(function(value, i){
            if(value.name === name){
                indexFieldGroupObject = i;
            } 
        });
        
        return indexFieldGroupObject;
        
    };
    
    /**
     * Procedimento para recuperar a posição
     * de um objeto dentro de um array.
     * 
     * @param {object} array array onde será feito
     * a varredura.
     * 
     * @param {object} objectName nome do objeto a ser
     * buscado.
    */
    this.getObjectPositionInArrayByName = function(array, objectName){
        
        var indexFieldObject;
      
        array.forEach(function(value, i){
            if(value.name === objectName){
                indexFieldObject = i;
            } 
        });
        
        return indexFieldObject;
        
    };
    
    /**
     * Limpa um field de acordo com seu template.
     * 
     * @param {object} field terá seu valor limpo
    */
    this.clearFieldValue = function(field){
        
        var templateField = field.template;
        
        if(templateField === "field/select.html"){
            
            if(field.value() && field.value() !== ""){
                
                field.clearByOutData();
            }
            
        }else{
            
            field.clearValue();
            
        }
        
    };
    
     /**
     * Função que habilita a criação de um novo código
     * para o registro que será salvo.
     * 
     * @param {object(widget)} widget widget ativa que
     * terá seu parâmetro alterados
     * 
     * @param {String} tableName tabela referência para
     * gerar o novo código
     * 
     * @param {String} tablePk nome da Primary Key da tabela
     * que recebera o código gerado.
    */
    this.enableNewNumericalCode = function(widget, tableName, tablePk){
        
        if(widget.currentRow.__is_new){
            
            var paramsNewCode = widget.paramsNewCode;
            
            paramsNewCode.__is_new_code = true;
            paramsNewCode.__new_code_pk = tableName;
            paramsNewCode.__table_pk = tablePk;
            
        }
        
    };
    
     /**
     * Define os parâmetros para gerar um novo código
     * para o registro a ser salvo.
     * 
     * @param {object} row objeto que será persistido.
     * 
     * @param {String} tableName tabela referência para
     * gerar o novo código
     * 
     * @param {String} tablePk nome da Primary Key da tabela
     * que recebera o código gerado.
     * 
     * @param {string} columnToReturn coluna a ser retornada.
    */
    this.setColumnToReturnParamsOnRow = function(row, tableName, tablePk, columnToReturn){
        
        row.__is_new_code = true;
        row.__new_code_pk = tableName;
        row.__table_pk = tablePk;
        row.__new_code_length = 20;
        row.__columns_to_return = [columnToReturn];
        
    };
    
    /**
     * Função que desabilita a criação de um novo código.
     * 
     * @param {object(widget)} widget widget ativa
     * que terá seus parâmetros desabilitados.
    */
    this.desableNewNumericalCode = function(widget){
    
        var paramsNewCode = widget.paramsNewCode;
        
        paramsNewCode.__is_new_code = false;
        paramsNewCode.__new_code_pk = "";
        paramsNewCode.__table_pk = "";
        
    };
    
    /**
     * Função para setar a coluna a ser retornada
     * na resposta do 'save'.
     * 
     * @param {object} currentRow currentRow que receberá
     * a propriedade com o nome da coluna a ser retornada.
     * 
     * @param {string} columnName nome da coluna a ser
     * retornada.
    */
    this.setColumnToReturnInCurrentRow = function(currentRow, columnName){
    
        currentRow.__columns_to_return = [columnName];

    };
    
    /**
     * Função para remover todos os caracteres
     * não numéricos de um determinado valor,
     * caso o valor recebido seja vazio, uma
     * String vazia é retornada.
     * 
     * @param {String} valueWithCurrencyMask valor que será retornado
     * sem caracteres não numéricos.
     * 
     * @return {String} valueWithoutSymbols valor
     * recebido como parâmetro sem caracteres não
     * numéricos.
     * 
    */
    this.removeValueMask = function(valueWithMask){
        
        if(valueWithMask){
        
            var valueWithoutSymbols = valueWithMask.replace(/[^0-9]/g, "");
            
            return valueWithoutSymbols;
        
        }else{
            
            return "";
            
        }
    };
    
    /**
     * Função para removar mascarás de moedas do valor
     * recebido como parâmetro.
     * 
     * @param {String} valueWithCurrencyMask valor que 
     * será retornado sem máscara de moeda.
     * 
     * @return {String} valueWithoutSymbols valor
     * recebido como parâmetro sem máscara de moeda.
     * 
    */
    this.removeValueCurrencyMask = function(valueWithCurrencyMask){
        
        if(valueWithCurrencyMask){
            if(typeof valueWithCurrencyMask === "string") {
                var valuesWithoutCurrencyMask = $.currency.unmaskString(valueWithCurrencyMask, {"thousands": "."});    
                return valuesWithoutCurrencyMask;
            }else{
                //Retorna o pŕoprio valor se não vier uma string
                return valueWithCurrencyMask;
            }
        }else{
            
            return "";
            
        }
        
            
    };
    
    /**
     * Função para localizar uma widget dentro
     * de um array de widgets pelo seu nome;
     * 
     * @param {object} widgets arrays de widgets, que
     * será percorrido pela busca.
     * 
     * @param {string} widgetName nome da widget a ser
     * procurada.
     * 
     * @return {object} foundWidget widget encontrada
     * no array de widgets recebido como parâmetro.
    */
    this.getWidgetByName = function(widgets, widgetName){
        
        var foundWidget;
        
        widgets.map(function(widget){
            
            if(widget.name === widgetName){
                foundWidget = widget;
            }
            else if(widget.widgets && widget.widgets.length > 0 && !foundWidget){
                
                foundWidget = this.getWidgetByName(widget.widgets, widgetName);
                
            }
            
        }.bind(this));
        
        return foundWidget;
        
    };
    
    /**
     * Função para setar o filtro do número da organização
     * padrão e do número da organização do operador logado
     * na field recebida como parâmetro, e logo em seguida,
     * recarregar a field.
     * 
     * @param {object} field field que receberá o filtro.
    */
    this.setDefaultOrgFilterAndOperatorOrgFilter = function(field){
        
        var fieldFilter = field.dataSourceFilter;
        var loggedOperatorInfos = this.getLoggedOperatorInfos();
        var valueWithDefaultOrgAndJobOrg = [getLocalVar("CRM_NRORGPADRAO"), loggedOperatorInfos.NRORG];
        
        if(!fieldFilter){
            
            field.dataSourceFilter = [];
            fieldFilter = field.dataSourceFilter;
            
        }
        
        var nrorgFilterIndex = this.getFilterPosition(fieldFilter, "NRORG");
        
        if(nrorgFilterIndex >= 0){
            fieldFilter[nrorgFilterIndex].value = valueWithDefaultOrgAndJobOrg;
        }else{
            fieldFilter.push(
                {
                    "name": "NRORG",
                    "operator": "IN",
                    "value": valueWithDefaultOrgAndJobOrg
                }
            );
        }
        
    };
    
    /**
     * Função para setar o filtro de organização padrão
     * para atribuir o valor do parâmetro 'P_NRORGPADRAO'
     * nas querys do dataSource em caso.
     * 
     * @param {object} filterDestination destino onde o filtro
     * será definido.
    */
    this.setDefaultOrgNumberFilterValue = function(filterDestination){
        
        var filterDestinationFilter = filterDestination.dataSourceFilter;
        
        if(!filterDestinationFilter){
            
            filterDestination.dataSourceFilter = [];
            filterDestinationFilter = filterDestination.dataSourceFilter;
            
        }
        
        filterDestinationFilter.push(
            {
                "name": "P_NRORGPADRAO",
                "operator": "IN",
                "value": [getLocalVar("CRM_NRORGPADRAO")]
            }
        );
        
    };
    
    /**
     * Função para remover um objeto do array(filtro)
     * recebido como parâmetro.
     * 
     * @param {object} filter array com os objetos(filtros).
     * 
     * @param {int} objectIndex índice do objecto a ser removido
     * do array(filtro).
    */
    this.deleteFilterObject = function(filter, objectIndex){
        
        if(objectIndex === 0){
            filter.shift();    
        }else if(objectIndex === filter.length - 1){
            filter.pop();
        }else{
            filter = filter.slice(0,objectIndex).concat(filter.slice(objectIndex + 1));    
        }
        
        return filter;
    };
   
    /**
     * Função para retornar a data e hora atual.
     * 
     * @return {string} data e hora atual.
    */
    this.getDataHoraAtual = function() {
        return moment().format('DD/MM/YYYY HH:mm');            
    };
    
    /**
     * Função para retornar a data atual.
     * 
     * @return {string} data atual.
    */
    this.getCurrentDate = function(){
        return moment().format('DD/MM/YYYY');
    };
    
    /**
     * Função para retornar a hora atual.
     * 
     * @return {string} hora atual.
    */
    this.getCurrentTime = function(){
        return moment().format('HH:ss');
    };
    
    /**
     * Função para retornar a data atual com
     * o horário máximo.
     * 
     * @return {string} data atual.
    */
    this.getCurrentDateWithMaxTime = function(){
        return moment().format('DD/MM/YYYY 23:59');
    };
    
    /**
     * Função para retornar a data atual com
     * o horário mínimo.
     * 
     * @return {string} data atual.
    */
    this.getCurrentDateWithMinTime = function(){
        return moment().format('DD/MM/YYYY 00:00');
    };
    
    /**
     * Procedimento para retornar uma data
     * subtraida pela quantidade de dias recebida
     * como parâmetro.
     * 
     * @param {string} date data s ser subtraida.
     * 
     * @param {number} numberOfDays dias como subtraendo.
     * 
     * @return {string} data subtraida.
    */
    this.subtractDateByDays = function(date,numberOfDays){
        return moment(date, 'DD/MM/YYYY').subtract(numberOfDays, 'day').format('DD/MM/YYYY');
    };
    
    /**
     * Procedimento para retornar uma data
     * somada pela quantidade de dias recebida
     * como parâmetro.
     * 
     * @param {string} date data s ser somada.
     * 
     * @param {number} numberOfDays dias como parcela.
     * 
     * @return {string} data somada.
    */
    this.sumDateByDays = function(date,numberOfDays){
        return moment(date, 'DD/MM/YYYY').add(numberOfDays, 'day').format('DD/MM/YYYY');
    };
    
    /**
     * Procedimento para retornar uma data
     * subtraida pela quantidade de mêses recebido
     * como parâmetro.
     * 
     * @param {string} date data s ser subtraida.
     * 
     * @param {number} numberOfMonth mêses como subtraendo.
     * 
     * @return {string} data subtraida.
    */
    this.subtractDateByMonth = function(date,numberOfMonth){
        return moment(date, 'DD/MM/YYYY').subtract(numberOfMonth, 'month').format('DD/MM/YYYY');
    };
    
    /**
     * Procedimento para retornar uma data
     * subtraida pela quantidade de mêses recebido
     * como parâmetro.
     * 
     * @param {string} date data s ser adicionada.
     * 
     * @param {number} numberOfMonth mêses como parcela.
     * 
     * @return {string} data adicionada.
    */
    this.sumDateByMonth = function(date,numberOfMonth){
        return moment(date, 'DD/MM/YYYY').add(numberOfMonth, 'month').format('DD/MM/YYYY');
    };
    
    /**
     * Adiciona um número de dias a uma data.
     * 
     * @param {string} date data se ser modificada.
     * 
     * @param {number} numberOfDays número de dias a ser
     * adicionado.
     * 
     * @param {string} format formato da data recebida.
     * 
     * @return  {string} data com os dias adicionado.
    */
    this.addDaysToDate = function(date, numberOfDays, format){
        return moment(date, format).add(numberOfDays, 'day').format(format);
    };
    
    /**
     * Retornar a data de uma string no formato
     * 'dateTime'.
     * 
     * @param {string} dateTime string no formato
     * 'dateTime'.
    */
    this.getDateInDateTime = function(dateTime){
        return moment(dateTime, 'DD/MM/YYYY HH:ss').format('DD/MM/YYYY');
    };
    
    /**
     * Retornar o horário de uma string no formato
     * 'dateTime'.
     * 
     * @param {string} dateTime string no formato
     * 'dateTime'.
    */
    this.getTimeInDateTime = function(dateTime){
        return moment(dateTime, 'DD/MM/YYYY HH:ss').format('HH:ss');
    };
    
    /**
     * Altera o formato da data de acordo com o
     * formato recebido como parâmetro.
     * 
     * @param {string} date data que será modificada.
     * 
     * @param {string} currentFormat formato atual da
     * data recebida.
     * 
     * @param {string} newFormat formato que será aplicado
     * na data recebida.
     * 
     * @return {string} data recebida formatada de acordo com
     * o novo padrão também recebido.
    */
    this.changeDateFormat = function(date, currentFormat, newFormat){
        return moment(date, currentFormat).format('YYYY-MM-DD');
    };
    
    /**
     * Função para filtrar os valores de um
     * campo de localização pelo seu local de origem
     * (Ex.:Minas Gerais (estado selecionado)
     * Cidades disponíveis: Contagem, Belo Horizonte, Betin).
     * 
     * @param {object} widget widget onde se localiza o campo
     * de localidade.
     * 
     * @param {object} field campo de localidade que receberá
     * o filtro pelo seu local de origem.
    */
    this.setPlaceFilterByPlaceOrigin = function(widget, field){
        
        var simplifiedFieldName = this.getPartOfTheExpressionFromLeftToRight(field.name, 4);
        var currentRow = widget.currentRow;
        var filters = [];
        var filterReference;
        
        switch(simplifiedFieldName){
            
            case "NMES":
                filterReference = "CDPAIS";
                filters.push(this.createFilterWithEqualityOperator(filterReference,currentRow.CDPAIS));
            break;
            
            case "NMMU":
                filterReference = "SGESTADO";
                filters.push(this.createFilterWithEqualityOperator(filterReference,currentRow.SGESTADO));
                filterReference = "CDPAIS";
                filters.push(this.createFilterWithEqualityOperator(filterReference,currentRow.CDPAIS));
            break;
            
            case "NMBA":
                filterReference = "CDMUNICIPIO";
                filters.push(this.createFilterWithEqualityOperator(filterReference,currentRow.CDMUNICIPIO));
                filterReference = "SGESTADO";
                filters.push(this.createFilterWithEqualityOperator(filterReference,currentRow.SGESTADO));
                filterReference = "CDPAIS";
                filters.push(this.createFilterWithEqualityOperator(filterReference,currentRow.CDPAIS));
            break;
            
        }
        
        if(filters){
            filters.forEach(function(filter){
                var placeNameFilterIndex = this.getFilterPosition(field.dataSourceFilter, filter.name);
            
                if(placeNameFilterIndex >= 0){
                    field.dataSourceFilter[placeNameFilterIndex].value = currentRow[filter.name];
                }else{
                    field.dataSourceFilter.push(filter);   
                }
            }.bind(this));
        }
    };
    
    /**
     * Função que recebe uma expressão e retorna
     * a mesma expressão simplicada(o número de 
     * caracteres recebido como parâmetro, começando
     * da esquerda para a direita.)
     * 
     * @param {string} string expressão a ser simplificada.
     * 
     * @para {number} numbersOfChars número de caracteres
     * que será retornado da expressão recebida com parâmetro.
    */
    this.getPartOfTheExpressionFromLeftToRight = function(expression, numbersOfChars){
        
        eval('var regEx = /(\\w{'+numbersOfChars+'})(\\w*)/');
        
        var newExpression = expression.replace(regEx, "$1");
        return newExpression;
    };
    
    /**
     * Função que retorna um 'objeto filtro'
     * com o operador de igualdade.
     * 
     * @param {string} name nome de referência para
     * o filtro.
     * 
     * @param {string/number} value valor buscado pelo
     * filtro.
    */
    this.createFilterWithEqualityOperator = function(name,value){
        
        var filter = {
            "name": name,
            "operator": "=",
            "value": value
        };
            
        return filter;
    };

    /**
     * Valida a inscrição informada pelo usuário
     * e seta na currentRow da widget o código do
     * tipo da inscrição e a máscara da inscrição,
     * caso a inscrição seja inválida
     * ele limpa os campos e manda uma mensagem.
     * 
     * @param {object} widget        : widget que possui a currentRow
     *                                  a ser modificada
     * 
     * @param {string} inscricao     : string com o nome do field 
     *                                  que possui a Inscrição
     * 
     * @param {string} tipoInscricao : string com o nome do field 
     *                                  do tipo de inscrição
    */
    this.validaInscricao = function(widget, inscricao, tipoInscricao){
        
        var currentRow = widget.currentRow;
        
        var resultValidate = General.inscricaoFormat(currentRow[inscricao]);
        
        if(resultValidate.valid && resultValidate.value){
            
            currentRow[tipoInscricao] = resultValidate.tipo.cod;
            
            currentRow[inscricao] = resultValidate.value;
            
        }else{
            
            ScreenService.showMessage('Número de inscrição inválido');
            
            currentRow[inscricao]     = "";
            currentRow[tipoInscricao] = "";
        }    
        
    };
    
    this.validaCPF = function(widget, inscricao){
        
        var currentRow = widget.currentRow;
        
        var resultValidate = General.validateCPF(currentRow[inscricao]);
        
        if(resultValidate){
            resultValidate = General.inscricaoFormat(currentRow[inscricao]);
            currentRow[inscricao] = resultValidate.value;
            
        }else{
            
            ScreenService.showMessage('CPF inválido');
            
            currentRow[inscricao]     = "";
        }    
        
    };
    
    /**
     * Busca as parametrizações do sistema
     * de acordo com o número da organização
     * do operador logado no template Manager.
     * 
     * @return {object} array com as parametrizações
     * da organização do operador logado.
    */
    this.getCrmParametrization = function(){
        
        return templateManager.CRM_PARAMETRIZACAO; 
      
    };
    
    /**
     * Esconde as ações funcionais do container
     * ativo.
    */
    this.hideFunctionalActions = function(){
        
        var containerActions = templateManager.container.actions;
        
        containerActions.forEach(function(action){
                
            if(action.label !== "Voltar"){
                action.isVisible = false;
                action.disable = true;
            }
            
        });    
    
    };
    
    /**
     * Retorna as configurações padrão das ações
     * do container ativo.
     * 
     * @param {object} widget em caso.
    */
    this.returnContainerDefaultActions = function(widget){
        
        var containerActions = templateManager.container.actions;

        containerActions.forEach(function(action){
            
            if(Boolean(action.disable)){
                action.isVisible = true;
                delete action.disable;
            }
            
        });
        
    };
    
    /**
     * Define o filtro(regra) nos fields definidos de acordo
     * com a revenda do operador logado.
     * 
     * @param {object} objects array com os objetos a receberem
     * a regra.
    */
    this.setResaleRulesInRestrictObjectsByLoggedOperatorResale = function(objects) {
        
        objects.forEach(function(field){
            this.setObjectFilterWhenResaleLoggedOperator(field);    
        }.bind(this));
       
    };
    
    /**
     * Defini os filtros de revenda do obejto recebido
     * como parâmetro (field || widget).
     * 
     * @param {object} objeto object a ser modificado.
    */
    this.setObjectFilterWhenResaleLoggedOperator = function(object) {
        
        if(this.isResaleLoggedOperator()) {
            
            this.changeObjectOperTypeFilterToResaleOper(object);
            this.setObjectResaleFilter(object);
        }
        
    };
    
    /**
     * Retorna um valor lógico apontando se o operador
     * logado pertence a uma revenda.
     * 
    */
    this.isResaleLoggedOperator = function(){
        
        var loggedOperatorInfos = this.getLoggedOperatorInfos();
        
        return (Boolean(loggedOperatorInfos.NRREVENDA) && loggedOperatorInfos.IDTPOPERADOR === 'R');
        
    };
    
    /**
     * Altera o filtro de tipo de operador do objeto
     * recebido como parâmetro.
     * 
     * @param {object} object objeto a ser modificado.
    */
    this.changeObjectOperTypeFilterToResaleOper = function(object) {
        
        var objectFilter = object.dataSourceFilter;
        var idtooperadorFilterIndex = this.getObjectPositionInArrayByName(objectFilter, 'IDTPOPERADOR');
        var resaleTypeOperator = 'R';
        
        if(idtooperadorFilterIndex === 0 || Boolean(idtooperadorFilterIndex)){
            objectFilter[idtooperadorFilterIndex].value = resaleTypeOperator;    
        }
      
    };
    
    /**
     * Define um filtro no objeto recebido como parâmetro
     * atribuindo o número da revenda o qual o operador
     * logado pertence.
     * 
     * @param {object} object objeto a ser modificado.
    */
    this.setObjectResaleFilter = function(object) {
        
        var loggedOperatorInfos = this.getLoggedOperatorInfos();    
        
        var resaleFilter = {
            name: "NRREVENDA",
            operator: "=",
            value: loggedOperatorInfos.NRREVENDA
        };
        
        object.dataSourceFilter.push(resaleFilter);
        
    };
        
    /* Função para confirmar e setar na currentRow 
     * do endereço em cadastro, o endereço retornado
     * da pesquisa pelo CEP no banco.
     * 
     * @param {string} CEP cep que será utilizado 
     *  para encontrar o endereço
     * 
    */
    this.buscarCep = function(CEP) {
        CEP = CEP.replace(/[\D]+/g,'');

        if(CEP.length == 8) {
            return restEngine.post('/findAddress', {
                    'requestType':'Row',
                    'row': {'CEP': CEP}
            });
        } else {
            ScreenService.showMessage('CEP Inválido.');
        }
    };
    
    /*
     * Deixa primeira letra das palavras de uma frase maíusculas
    */
    this.capitalizeFirstLetter = function(string){
        string = string.split(' ');
        string.forEach(function(partOfString, index, string){
            partOfString = partOfString.charAt(0).toUpperCase() + partOfString.toLowerCase().slice(1);    
            string[index] = partOfString;
        });
        return string.join(' ');
    };
    
    /*
     * registra a validação do tipo 'phone'
    */
    this.registerPhoneValidation = function(){
        
        var phoneValidation = function(value, validationAttrs) {
            var retorno;
            if(value === null || value === undefined){
                retorno =  {"valid":true};
            }else{    
                var numberSizeReceived = value.replace(/[^0-9]/g,'').length;
                retorno = {
                    "valid": (numberSizeReceived === 10 || numberSizeReceived === 11 || numberSizeReceived === 12 || numberSizeReceived === 0),
                    "message": ScreenService.i18n(validationAttrs.message) || ScreenService.i18n("Invalid phone.")
                };    
            }
            return retorno;
        };
            	      
        ValidationEngine.registerValidation('phone', phoneValidation);
    };
    
    this.registerMinCurrencyValidation = function(){
        
        var minCurrency = function(value, validationAttrs) {
            var limit = parseFloat(validationAttrs.limit);
            if(value) {
                value = parseFloat(value.replace(/[^0-9]/g, '').slice(0, -2));
            } else {
                value = 0;
            }
    		
    		return {
    			"valid": limit <= value || (isNaN(limit) && isNaN(value)),
    			"message": ScreenService.i18n(validationAttrs.message) || ScreenService.i18n("Minimum value: ") + validationAttrs.limit
    		};
        }; 
            	      
        ValidationEngine.registerValidation('minCurrency', minCurrency);
    };
    
    /**
     * Inicializa as actions da widget e suas 
     * filhas recursivamente.
     * 
     * @param {object} widget widget de evolução.
     * 
    */    
    this.factoryWidgetActions = function(widget){
        if(!Util.isEmptyOrBlank(widget.widgets)){
            widget.widgets.forEach(function(inWidget){
                this.factoryWidgetActions(inWidget);
            }.bind(this));
        }
        widget.actions.forEach(function(action){
           metaDataFactory.actionFactory(action, widget);
        });
        widget.activate();
    };
    
    /**
     * Seta o datasource filter das tabs dinamicas 
     * baseado num campo da widget principal.
     * 
     * @param {object} widget widget dinamica
     * @param {object} widget widget principal
    */
    this.setDataSourceFilterOnDynamicTabs = function(tabWidget, activeWidget){
        
        var NRSEQCONTATO = activeWidget.currentRow.NRSEQCONTATO;
        tabWidget.dataSourceFilter[0].value = NRSEQCONTATO;
        if(Util.isEmptyOrBlank(tabWidget.currentRow.NRSEQCONTATO))
            tabWidget.currentRow.NRSEQCONTATO = NRSEQCONTATO;
        tabWidget.reload();
        
    };
    
    /**
     * Seta um elemento em uma posição definida dentro de um
     * array.
     * 
     * @param {all} element elemento a ser inserido no array.
     * 
     * @param {object} array array a ser modificado.
     * 
     * @param {int} position posição que o elemento entrará no
     * array.
     * 
     * @return {object} array modificado.
    */
    this.setElementOnArrayDefinedPosition = function(element, array, position){
        
        if(!Boolean(position)){
            array.push(element);
        } else {
            array.splice(position - 1, 0, element);    
        }
        
        return array;
        
    };
    
    /**
     * Verifica se a primeira data recebida como parâmtro
     * é maior que a segunda data, também recebida como
     * parâmetro.
     * Valida um intervalo de datas.
     * 
     * @param {string} initDate data inicial do intervalo.
     * 
     * @param {string} endDate data final do intervalo.
     * 
     * @param {string} format formato das datas recebidas.
     * 
     * @retur {boolean} valor lógico indicando se a primeira
     * data é maior ou não que a segunda.
    */
    this.validateIntervalDates = function(initDate, endDate, format){
        
        if(moment(initDate, format).format(format) > moment(endDate, format).format(format)){
            return false;
        }
        
        return true;
    };
    
    /**
     * Define a propriedade 'readOnly' do campo recebido como
     * parâmetro.
     * 
     * @param {object} field campo a ser modificado.
     * 
     * @param {boolean} isReadOnly valor lógico definindo a 
     * propriedade do campo.
    */
    this.setFieldReadOnlyProperty = function(field, isReadOnly){
        field.readOnly = isReadOnly;  
    };
    
    /**
     * Aplica o método de inserção na widget recebida como parâmetro e
     * seta na currentRow a coluna retornada.
     * 
     * @param {object} widgtet widget que sofrerá a rotina de inserção.
     * 
     * @param {string} columnToReturn coluna a ser retornada na currentRow.
    */
    this.saveAndSetColumnToBeRetunrOnCurrentRow = function(widget, columnToReturn){
        
        var currentRow = widget.currentRow;
        var newRow = currentRow.__is_new;
        
        this.setColumnToReturnInCurrentRow(currentRow, columnToReturn);
        
        return General.widgetSave(widget).then(function(response){
            if (newRow) {
                this.reloadGridInViewMode(widget);
            }
            
            currentRow[columnToReturn] = response.dataset.columnsToReturn[0][columnToReturn];    
        });
        
    };
    
    /**
     * Sobreescreve a função relacionada a uma ação
     * da widget recebida como parâmetro.
     * 
     * @param {object} widget widget que terá sua ação
     * modificadas.
     * 
     * @param {string} actionName nome da ação a ser 
     * modificada.
     * 
     * @param {string} functionCall chamada da função a ser colocada
     * na action.
     * EX.: ExemploController.exmploFuncao();
     * Vale lembrar que a chamada da função é atribuida no JSON da
     * widget, sendo assim, os parâmetros devem seguir as regras de
     * chamdas em JSON.
    */
    this.overwriteAction = function(widget, actionName, functionCall){
        
        if(!Boolean(widget.getAction(actionName).events[0].defaultCode)){
            widget.getAction(actionName).events[0].defaultCode = widget.getAction(actionName).events[0].code;
        }
        
        widget.getAction(actionName).events[0].code = functionCall;
        
    };
    
    /**
     * Retorna a função padrão de uma ação (quando a mesma
     * foi sobrescrita).
     * 
     * @param {object} widget widget que terá sua ação 
     * modificada.
     * 
     * @param {string} actionName nome da ação a ser 
     * modificada.
    */
    this.setActionDefaultCode = function(widget, actionName){
        
        widget.getAction(actionName).events[0].code = widget.getAction(actionName).events[0].defaultCode;
        
    };
    /* Função que verifica se o caractere digitado é do tipo NUMBER ou STRING,
    caso seja STRING, substitui o valor para vazio, caso seja numero, passa o valor digitado*/ 
    this.onlyNumbers = function(field) {
		const fieldValue = field.value();
		if (fieldValue) {
			var fieldElements = fieldValue.split("");
			fieldElements.forEach((value, index) => {
				if (isNaN(parseInt(value))) {
					fieldElements.splice(index,1);
					field.value(fieldElements.join(''));
				}
			});
		}
	};
	
	this.changeVisibilityOrgTrab = function(nrOrgRegistro, widget, actions, fields) {

        var visible = true;
        var nrOrgTrab = getLocalVar("CRM_NRORGTRAB");

        if (nrOrgRegistro == nrOrgTrab || widget.currentRow.__is_new) {
            visible = true;
        }
        else {
            visible = false;
        }

        if (!Util.isUndefinedOrNull(actions)) {
            actions.forEach(function(action) {
                if (widget.getAction(action)) {
                    widget.getAction(action).isVisible = visible;
                }
            });
        }

        if (!Util.isUndefinedOrNull(fields)) {
            fields.forEach(function(field) {
                if (widget.getField(field)) {
                    widget.getField(field).readOnly = !visible;
                }
            });
        }
    };
    
    /**
     * Faz com que o select-native respeite o outData, setando 
     * os capos na currentRow
    */
    this.outDataNativeFields = function(args){
        var field      = args.owner.field;
	    field.dataSource.data.forEach(function(option){
	        if(option[field.valueField] === args.row[field.name]){
	            //Limpa a antiga row
	            Object.keys(field.outData).forEach(function(key){
                    if(args.row[field.outData[key]] !== undefined) delete args.row[field.outData[key]];    	                     
	            });
	            Object.keys(option).forEach(function(key){
	                if(field.outData[key] !== undefined){
	                    args.row[field.outData[key]] = option[key];
	                }
	            });
	        }      
	    });
	};
	
	this.applyFieldDefaultValueOnFilterPopup = function(filterWidget, field) {

        var fieldDefaultValue;

        if (!Util.isEmptyOrBlank(field.filterProperties) && field.filterProperties.hasOwnProperty("defaultValue")) {
            fieldDefaultValue = field.filterProperties.defaultValue;
        } else {
            fieldDefaultValue = field.defaultValue;
        }

        if (filterWidget.currentRow) {
            filterWidget.currentRow[field.name] = typeof fieldDefaultValue === "undefined" ? "" : ZhUtil.evaluate(fieldDefaultValue);
        }
    };
    
    this.openFilterOnEnter = function(widgetToFilter, row, fieldsToApplyDefaultValue = []) {
        var filterRow = row || {};
        fieldsToApplyDefaultValue.forEach(function(fieldName) {
            let field = widgetToFilter.getField(fieldName);
            if (field) {
                this.applyFieldDefaultValueOnFilterPopup(widgetToFilter, field);
                filterRow[fieldName] = field.value();
            }
        }.bind(this));
        ZhFilterPreferences.setOriginalFilter(widgetToFilter.id, filterRow);
        return ZhFilterPreferences.openFilterWidget(widgetToFilter, filterRow).then(function() {
            return templateManager.container.getActiveWidget();
        });
    };
    
    /**
     * Converts a date variable from YYYY-MM-DD to DD/MM/YYYY format.
     * 
     * @param {string} date - The date(YYYY-MM-DD) to be converted.
     * @returns {string} Converted date (null if the given date is invalid).
     * 
     * @author Reynaldo Pinto <reynaldo.pinto@teknisa.com>
     */
    this.convertDateFromFilter = function(date) {
        if(typeof date == 'string') {
            var dateSplit = date.split('-');
            if(dateSplit.length == 3) {
                var year    = dateSplit[0];
                var month   = dateSplit[1];
                var day     = dateSplit[2];
                
                var newDate = `${day}/${month}/${year}`;
                
                return newDate;
            }
            
        }
        
        return null;
    }
    
    /**

     * Applies the readOnly property to the fields in a cascade-like way.
     * That is, a field is only writable when its "parent" field has a valid value.
     * 
     * @param {Widget} widget - Widget in which the fields are.
     * @param {string[]} fieldsNames - Fields to have the cascade property applied.
     * 
     * @author Reynaldo Pinto <reynaldo.pinto@teknisa.com>
     */
    this.cascadeReadOnly = function(widget, fieldsNames) {
        var fields = [];
        
        // Get the respective fields from the widget using their names
        fieldsNames.forEach(function(name) {
            var field = widget.getField(name);
            fields.push(field);
        });
        
        // It doesn't make any sense to apply it if there's only one field
        if(fields.length > 1) {
            fields.forEach(function(field, index, fields) {
                
                //Isn't the last field (has a child field)
                if(index < fields.length - 1) {
                    // Makes the next field its child
                    field.childFieldName = fieldsNames[index + 1];
                }
            });
        }
    };
    
    /**
     * Function to be executed when a parent field's value changes.
     * It will will always clean its descendants' values, but it will only make them
     * read-only if the parent's new value is empty.
     * 
     * @param {Field} parentField - Field which owns a child field.
     * 
     * @author Reynaldo Pinto <reynaldo.pinto@teknisa.com>
     */
    this.parentFieldOnChange = function(parentField) {
        var childField = this.getFieldChild(parentField);
        
        // If a field's new value is empty, its descendants will be read-only.
        if(Util.isEmptyOrBlank(parentField.value())) {
            this.cascadeCleanValue(parentField, true);
        } else {
            this.cascadeCleanValue(parentField, false);
            childField.readOnly = false;
        }
    };
    
    /**
     * Cleans (makes empty) the value of all field's "descendants" (cascade).
     * 
     * @param {Field} parentField - Field which will have all descendants' values
     *  cleaned.
     * @param {boolean} readOnly - Defines if the descendants will be read only.
     * 
     * @author Reynaldo Pinto <reynaldo.pinto@teknisa.com>
     */ 
    this.cascadeCleanValue = function(parentField, readOnly) {
        var childField = this.getFieldChild(parentField);
        var widget = parentField.widget;
        
        // Loops while the current field has a child
        while(!Util.isEmptyOrBlank(childField)) {
            
            var childName = childField.valueField || childField.name;
            
            // Cleans child's value
            widget.currentRow[childName] = undefined;
            childField.value(null);
            
            if(readOnly) {
                childField.readOnly = true;
            }
            
            // Gets the next child (child's child)
            childField = this.getFieldChild(childField);
        }
    };
    
    /**
     * Gets the child of the given field through its widget.
     * 
     * @param {Field} parentField - Field that owns the child to be returned.
     * @returns {Field} - Parent filter's child.
     * 
     * @author Reynaldo Pinto <reynaldo.pinto@teknisa.com>
     */
    this.getFieldChild = function(parentField) {
        var widget = parentField.widget;
        var childName = parentField.childFieldName;
        var childField = widget.getField(childName);
        
        return childField;
    };
    
    /**
     * Defines a property of a field in both general and filter properties.
     * 
     * @param {Field} field - Field which will have the property applied.
     * @param {string} propertyName - Property's name to be applied.
     * @param value - Property's new value.
     * 
     * @author Reynaldo Pinto <reynaldo.pinto@teknisa.com>
     */
    this.applyProperty = function(field, propertyName, value) {
        field[propertyName] = value;
        var filter = field.filterProperties;
        
        if(!filter) {
            field.filterProperties = filter = {};
        }
        
        filter[propertyName] = value;
    };
    
    /**
     * Returns the component's (Widget or Field) data source filter with the corresponding given name.
     * 
     * @param {object} component - Component in which the data source filter (hopefully) is.
     * @param {string} dsfName - Data source filter's name.
     * @returns {DataSourceFilter} - Data source filter with the corresponding name.
     * 
     * @author Reynaldo Pinto <reynaldo.pinto@teknisa.com>
     */
    this.getComponentDataSourceFilter = function(component, dsfName) {
        
        //Returns the first data source filter with the given name
        var filter = component.dataSourceFilter.find(function(dsFilter) {
            return dsFilter.name == dsfName;
        });
        
        return filter || undefined;
    };
    
    /**
     * Search and returns all fields with the given names in the widget.
     * 
     * @param {Widget} widget - Widget to look for the fields.
     * @param {string[]} names - The names of the fields to be returned.
     * 
     * @returns {Field[]} - Array of fields with the respective given names
     *  (undefined item if the field is not found).
     * 
     * @author Reynaldo Pinto <reynaldo.pinto@teknisa.com>
     */
    this.getWidgetFieldsByName = function(widget, names) {
        var fields = [];
        
        names.forEach(function(name) {
            var field = widget.getField(name);
            fields.push(field);
        });
        
        return fields;
    };
    
    /**
     * Agrupa um array de objetos de forma semelhante ao GROUP BY do SQL.
     * 
     * @param {Array} rows - Array de objetos(rows) a serem agrupados.
     * @param {string[]} columnsToGroup - Nome das colunas que serão utilizadas
     *como base para o agrupamento.
     */
    this.groupBy = function(rows = [], columnsToGroup = []) {
        
        // Caso não seja passado nenhum valor nas rows, será retornado uma
        //estrutura padrão vazia
        if(rows.length == 0) {
            var _default = {
                value: {},
                rows: []
            };
            
            return _default;
        }
        
        // Cria os grupos para cada row.
        var groups = rows.reduce(function(groupedRows, row) {
            // Verifica se já existe algum grupo com os mesmos valores da row
            //em questão.
            var group = groupedRows.find(function(gpdRow) {
                // Faz a verificação abaixo com todas as columas a serem
                //agrupadas.
                return columnsToGroup.reduce(function(sameGroup, column) {
                    // Se o valor da coluna em questão da row é igual ao do
                    //grupo sendo verificado.
                    sameGroup &= (row[column] === gpdRow.value[column]);
                    
                    return sameGroup;
                }, true);
            });
            
            // Caso não ache um grupo com esses valores, criará um novo.
            if(group === undefined) {
                group = this.createNewGroup(row, columnsToGroup);
                groupedRows.push(group);
            }
            
            group.rows.push(row);
            
            return groupedRows;
        }.bind(this), []);
        
        return groups;
    };
    
    this.createNewGroup = function(row, groupColumns) {
        var group = {
            value: {},
            rows: []
        };
        
        group.value = groupColumns.reduce(function(groupValue, column) {
            groupValue[column] = row[column];
            
            return groupValue;
        }, {});
        
        return group;
    };
    
    this.saveWidget = function(widget, closeSwipe = true) {
        // Assumindo que a rota para salvar é a mesma que para pegar os
        //dados + /save
        var route = widget.dataSource.name + '/save';
        
        var params = {
            requestType: 'DataSet',
            dataset: [widget.currentRow]
        };
        
        // Envia uma requisição ao servidor para salvar a nova row
        var request = restEngine.post(route, params).promise;
        
        request.then(function(response) {
            // Sai do swipe de adicionar e vai para a grid
            if(closeSwipe) {
                ScreenService.closeSwipe();
            }
            
            // Ativa as tabs da widget
            widget.widgets.forEach(function(tab) {
                tab.setEnabled(true);
            });

        }).catch(this.showErrorMessage); // Mostra uma mensagem na tela caso haja algum erro
        
        // Retorna a promise, permitindo que ela seja tratada posteriormente em
        //outra função
        return request;
    };
    
    this.showErrorMessage = function(error) {
        ScreenService.showMessage(Messages.CATCH_EXCEPTION_ERROR(error.data.error));
    };
    
    this.reloadWithTempFilter = function(widget, columnName, columnValue) {

        // Criação do filtro que servirá pra mostrar apenas a row desejada
        var filterNewRow = {
            name: columnName,
            operator: '=',
            value: columnValue,
            // Indicação para saber qual filtro remover após
            //o reload
            temp: true 
        };
        
        // Adiciona o novo filtro criado ao data source filter
        widget.dataSourceFilter.push(filterNewRow);
        
        // Recarrega os dados e remove o filtro temporário
        return widget.reload().then(function() {
            widget.dataSourceFilter = widget.dataSourceFilter
                .filter(x => x.temp !== true)
        });
    }
    
    /**
     * Recarrega a widget passada apenas se seu filtro já tiver sido aplicado.
     * 
     * @param widget - Widget a ser verificado e recarregada.
     * @author Reynaldo Pinto <reynaldo.pinto@teknisa.com>
     */
    this.reloadWidgetIfFiltered = function(widget) {
        if(widget.filtered) {
            widget.reload();
        }
    };
    
    this.setWidgetAsFiltered = function(widget) {
        widget.filtered = true;
    };
    
    this.isWidgetFilter = function(widget) {
        var isFilter = widget.name == "zhFilterPreferencesSelectConditions";
        
        return isFilter;
    };
    
    this.post = function(uri, params, callback) {
        if(!uri || !params) {
            return undefined;
        }
        
        return restEngine.post(uri, params).then(response => {
            if(!response.dataset) {
                ScreenService.showMessage(Messages.CATCH_EXCEPTION_ERROR(response));
                throw response;
            } else {
                if(callback) {
                    callback(response);
                }
            }
            
            return response;
        }).catch(err => {
            this.showErrorMessage(err);
            throw err;
        });
    };
    
    this.getDataSourceFilter = function(widget, filterName) {
        var filter = widget.dataSourceFilter.find(function(dsFilter) {
            return dsFilter.name === filterName;
        });
        
        return filter;
    };
    
    this.reloadGridInViewMode = function(widget) {
        widget.mode = 'list';
        var promise = widget.reload();
        widget.mode = 'view';
        
        return promise;
    }
}

Configuration(function(ContextRegister) {
    ContextRegister.register('CRM_Utilities', CRM_Utilities);
});