/* global Util, UtilDevTools, ShowInDevTools, $ */
function DevToolsController($scope, $element, ContextRegister, ScreenService, templateManager, eventAggregator, ShortcutDeveloperService, DevToolsService, DeveloperTreeService){
	var self = this;
	var ENTER_KEY = 13;
	var DEVTOOLS_PACKAGE_NAME = 'devtools';
	var MainWindow = UtilDevTools.getMainWindow();

	ContextRegister.register('DevToolsController', this);

	function hasDevTemplateUrl(templates){
		return templates.some(function(template){
			return template.package == DEVTOOLS_PACKAGE_NAME;
		});
	}

	function setCursorPositionInActiveItem(){
		var item = $scope.activeFolder || DevToolsService.getLocalActiveItem();
		var cursorPosition = $scope.currentEditor ? $scope.currentEditor.getCursorPosition() : {};
		DevToolsService.setLocalActiveItem(item.name, item.path, item.mode, item.parent, cursorPosition);
	}

	if(!hasDevTemplateUrl(templateManager.templateUrls)){
		templateManager.templateUrls.push({
			"baseUrl":"./bower_components/devtools/templates/",
			"package": DEVTOOLS_PACKAGE_NAME
		});
	}

	var recoverComponentId = UtilDevTools.buildInCustomizationModeMethod(function(item, state){
		if(!$scope.inFileMode()){
			var text = JSON.parse(state.value);
			text.id = item.id;
			state.value = JSON.stringify(text);
		}
	});


	this.inCustomizationMode = function(){
		return templateManager.inCustomizationMode;
	};

	 this.getTemplateURLHeader = function(){
		var template = this.inCustomizationMode() ? 'devtools#/dt-lite.html' : 'devtools#/dt-default.html';
		return templateManager.getTemplateURL(template);
	};

	this.setUnloadedItems = function(data){
		DeveloperTreeService.setUnloadedItems(data.mode, data.items, data.setItems);
	};

	this.getUnloadedItems = function(mode){
		return DeveloperTreeService.getUnloadedItems(mode);
	};

	this.newFile = function(obj){
		MainWindow.executeFunction("DTComponentCRUD", "newFile", [obj]);
	};

	this.hideCreateFile = function(){
		$scope.hideCreateFile();
	};

	this.addNewContainer = function(){
		$scope.onCreateFile();
	};

	this.changeHeightOfCodeEditor = Util.buildDebounceMethod(function(open){
		var code = $element.find('.code');
		if(code && code[0]){
			var heightValue = $scope.isToSearchText() ? "80px": "50px";
			var hasShortcut = ($scope.isToSearchText() || $scope.isToCreateFile());
			open = open !== undefined ? open : hasShortcut;
			if(!open){
				heightValue = "0px";
				templateManager.updateTemplate();
				$scope.currentEditor.focus();
			}
			code[0].style.height = "calc(100% - " + heightValue + ")";
		}
	}, 100);

	this.openShortcutWidget = function(widgetName){
		var activeWidget = $scope.widget;
		if(activeWidget && activeWidget.name == widgetName){
			$scope.widget.reload();
		} else {
			if(activeWidget){
				ShortcutDeveloperService.hideWidget(activeWidget);
			}
			var widgetAndContainer = ShortcutDeveloperService.getWidgetAndContainerToShow(widgetName);
			$scope.widget = widgetAndContainer.widget;
			$scope.templateManager.container = widgetAndContainer.container;
		}
	};

	this.shouldShowShortcutWidget = function(){
		return $scope.isToSearchText() || $scope.isToFindLine() || $scope.isToCreateFile();
	};

	this.changeCaseSensitive = function(){
		var caseSensitive = $scope.widget.getField('caseSensitive');
		caseSensitive.value(!caseSensitive.value());
		caseSensitive.class = caseSensitive.value() ? "01 zh-dvtools zh-case-sensitive" : "01 zh-dvtools";
		$scope.findTextInEditor();
	};

	this.findTextInEditor = function(args, option){
		$scope.findTextInEditor(option, args.e.keyCode);
	};

	this.findLine = function(args){
		$scope.findLine(args);
	};

	var isEmptySelection = function(selection){
		var selectionDiff;
		if(selection.end && selection.start){
			selectionDiff = selection.end.column - selection.start.column || selection.end.row - selection.start.row;
		}
		return selectionDiff;
	};

	var replaceInEditor = function(callback){
		callback = callback || function(){};
		var replaceWith = $scope.widget.getField('textToReplace').value();
		var selection = $scope.currentEditor.selection.getRange();
		if(replaceWith && replaceWith.length > 0  && isEmptySelection(selection)){
			$scope.currentEditor.session.replace(selection, replaceWith);
			$scope.markTextInTextEditor("next", undefined);
			callback();
		}
	};

	this.replaceText = function(){
		replaceInEditor();
	};

	this.replaceAllText = function(){
		var callback = self.replaceAllText;
		replaceInEditor(callback);
	};

	this.updateDependencyFieldsVisibility = function(widget, fieldChanged){
		eventAggregator.publish('updateDependencyFieldsVisibility', widget.getField(fieldChanged));
	};

	this.updateActiveItemInEditor = function(paths) {
		$scope.updateActiveItem(paths);
	};

	this.setErrorsValidationInTree = function(errorsObject) {
		$scope.setInvalidItems(errorsObject);
	};

	this.addErrorValidationInTree = function(errorObject) {
		$scope.addInvalidItem(errorObject);
	};

	this.clearErrorsValidationInTree = function() {
		$scope.clearInvalidItems();
	};

	this.reloadErrorsValidationInTree = function(){
		$scope.reloadErrorsValidationInTree();
	};

	this.getActiveTreeItem = function(){
		return $scope.getActiveItem();
	};

	this.toggleInspectionMode = function() {
		window.inspection = !window.inspection;
	};

	this.alternateTree = function() {
		window.hide = !window.hide;
	};

	this.switchWindowMode = function(){
		setCursorPositionInActiveItem();
		MainWindow.changeWindowMode();
	};

	this.openPath = function(path, mode, cursorPosition) {
		$scope.openPath(path, self, cursorPosition);
		if(mode){
			self.changeCurrentMode(mode);
		}
	};

	this.getOpenFormImageBasedOnFormVisibility = function(){
		return $scope.inFormMode() ? 'json-view' : 'form-view';
	};

	this.getArrowImageBasedOnTreeVisibility = function(){
		return window.hide ? 'show-tree' : 'hide-tree';
	};

	this.getTreeTemplateUrlBasedOnEditorMode = function(){
		var treeItemHtml = $scope.inFileMode() ? 'developer/tree-item-project.html' : 'developer/tree-item.html';
		return templateManager.getFrameworkTemplateURL(treeItemHtml);
	};

	this.getShortcutUrl = function(){
		var shortcutHtml = 'developer/zh-search.html';
		if($scope.isToFindLine()){
			shortcutHtml = 'developer/zh-find-line.html';
		} else if($scope.isToCreateFile()){
			shortcutHtml = 'developer/zh-create-file.html';
		}
		return templateManager.getFrameworkTemplateURL(shortcutHtml);
	};

	this.getEditorTemplateUrlBasedOnEditorMode = function(){
		var codeEditorHtml = 'developer/code-editor-json.html';
		if($scope.inFileMode()){
			codeEditorHtml = 'developer/code-editor-js.html';
		} else if($scope.inFormMode()){
			codeEditorHtml = 'developer/form-editor.html';
		}
		return templateManager.getFrameworkTemplateURL(codeEditorHtml);
	};

	this.saveComponentChange = function() {
		var activeItem = $scope.getActiveItem();
		var state = activeItem ? $scope.getActiveComponentState() : { valid: false};
		if (state.valid){
			var componentObject = {};
			var indexPath = activeItem._indexPath;
			var fileName = indexPath ? indexPath.substring(indexPath.lastIndexOf("/")) : "";
			var activePath = $scope.getActiveComponentPath();
			activePath = activePath && activePath.length ? activePath : undefined;
			if($scope.inFileMode()){
				componentObject.data = $scope.lastScript;
			} else if($scope.inMenuMode()){
				var ZEEDHI_PROJECT = "zeedhi_project";
				activePath = activePath.indexOf(ZEEDHI_PROJECT) < 0 ? [ZEEDHI_PROJECT].concat(activePath) : activePath;
				componentObject.zeedhi_project = Util.clone(window.groupMenu.containers.zeedhi_project);
			} else {
				componentObject = Util.clone(window.currentContainer.__metadata);
				componentObject = ShowInDevTools.clear(componentObject);
			}
			recoverComponentId(activeItem.item, state);
			componentObject.fileName = fileName;
			componentObject.__indexPath = indexPath;
			var saveParams = [state.value, componentObject, activePath, $scope.inFileMode()];
			MainWindow.executeFunction("DTComponentCRUD", "saveComponentChange", saveParams);
		}
	};

	this.saveNewComponent = function(defaultJsonValue, activeItemPath, completeJson){
		var saveNewParams = [defaultJsonValue, activeItemPath, completeJson];
		MainWindow.executeFunction("DTComponentCRUD", "saveNewComponent", saveNewParams);
	};

	this.deleteComponent = function(activeItemPath, completeJson, deleteType){
		var deleteParams = [activeItemPath, completeJson, deleteType];
		MainWindow.executeFunction("DTComponentCRUD", "deleteComponent", deleteParams);
	};

	this.deleteItemCustomization = function(){
		var itemId = $scope.getActiveItem().item.id;
		MainWindow.executeFunction("DTComponentCRUD", "deleteComponentCustomization", [itemId]);
	};

	this.deleteCustomFile = function(){
		var fileName = $scope.activeItem.item.name;
		MainWindow.executeFunction("DTComponentCRUD", "deleteCustomFile", [fileName]);
	};

	this.resetContainer = function(){
		MainWindow.executeFunction("DTComponentCRUD", "resetContainer");
	};

	this.searchCustomization = function(){
		MainWindow.executeFunction("DTCustomizationService", "renderCustomizationInMainWindow", window.currentContainer);
	};

	this.inspectSentComponent = function(inspectData){
		var mode = inspectData.mode == 'form' ? 'json' : inspectData.mode;
		if(mode){
			self.changeCurrentMode(mode);
		}
		self.openPath(inspectData.componentPath, mode, inspectData.cursorPosition);
	};

	this.changeCurrentMode = function(mode){
		$scope.setCurrentMode(mode, self);
	};

	this.clearActiveItem = function(){
		$scope.activeItem = undefined;
	};

	this.showNotificationInDevTools = function(messageObj) {
		ScreenService.notificationMessage(messageObj.message, messageObj.type);
	};

	this.receiveMessages = function(message){
		if(message.functionName){
			return this[message.functionName](message.args);
		}
	};

	this.createNewFile = function(key){
		if(!key || key === ENTER_KEY){
			$scope.createNewFile();
		}
	};

	this.toggleDevMenu = function(event){
		document.getElementById("devToolsMenu").classList.toggle("showDevMenu");
		event.stopPropagation();
	};

	this.hideDevMenu = function() {
		document.getElementById("devToolsMenu").classList.remove("showDevMenu");
	};

	function hasButtonMenu(element){
		return element.closest("#devToolsButtonMenu").length;
	}

	this.clickOutMenu = function(event){
		if(!hasButtonMenu($(event.target))){
			self.hideDevMenu();
		}
	};

	if(!self.inCustomizationMode()){
		$(document).on("click", this.clickOutMenu);
		$scope.$on("$destroy",function(){
			$(document).off("click", this.clickOutMenu);
		});
	}

	$scope.$watch(
		function() { return window.hide; },
		function(newValue) {
			$scope.inHideMode = newValue;
		}
	);

	$scope.$watch(
		function() { return window.inspection; },
		function(newValue) {
			$scope.inInspectionMode = newValue;
		}
	);
}