var ZeedhiDirectives = angular.module("ZeedhiDirectives");

ZeedhiDirectives.directive('zhInspection', function($timeout, eventEngine, ApplicationContext, $window, eventAggregator, ElementToMetadataService) {
	return {
		link: function($scope, $element, $attrs, $controller) {
			var DEVELOPER_INSPECTION_MODE_CLASS = 'developer-inspection';

			var getBody = function(){
				return $window.document.body;
			};

			var addEditingClass = function(element) {
				$(element).addClass(DEVELOPER_INSPECTION_MODE_CLASS);
			};

			var removeEditingClass = function(element) {
				$(element).removeClass(DEVELOPER_INSPECTION_MODE_CLASS);
			};

			var getWindowController = function() {
				return ApplicationContext.WindowController;
			};

			var sendElementPathToDevTools = function(scope){
				var pathToMetadata = ElementToMetadataService.getPathToMetadata(scope);
				getWindowController().sendToDevTools(pathToMetadata, 'json');
			};

			var openWidgetToEdit = function(elem){
				var body = getBody();
				var $scope = angular.element(elem.target).scope();
				body.removeEventListener('click', openWidgetToEdit);
				sendElementPathToDevTools($scope);
				eventEngine.restoreDispatchEvent();
				removeEditingClass(body);
			};

			var enableEditMode = function(){ 
				var body = getBody(); 
				$timeout(function(){ body.addEventListener("click", openWidgetToEdit); });
				addEditingClass(body);
				eventEngine.stopDispatchEvent("Entering in edit mode.");
			};

			var disableEditMode = function(){
				var body = getBody();
				body.removeEventListener('click', openWidgetToEdit);
				removeEditingClass(body);
				eventEngine.restoreDispatchEvent();
			};

			var runDeveloperEditMode = UtilDevTools.buildMainWindowMethod(function(args){
				window.inspection = !window.inspection;
				eventAggregator.publish('developerEnableEditMode', {'enableEditMode': window.inspection});
			});			
			Mousetrap.bind('mod+f11', runDeveloperEditMode);

			eventAggregator.subscribe('developerEnableEditMode', function(message){
				var enable = message ? message.enableEditMode : true;
				if(enable){
					enableEditMode();
				} else {
					disableEditMode();
				}
			});
		}
	};
});