/*global diff, Util */
function ElementToMetadataService(templateManager) {
    var self = this;
    var devToolsWindow = window.opener || window.parent;
    var MENU_TYPE = "menu";
    var GROUP_MENU_TYPE = "groupMenu";
    var CONTAINER_TYPE = "container";
    var PARENT_MENU = "parentMenu";
    var DELETE = "DELETE";
    var NEW = "NEW";
    var DELETEWIDGET = "DELETEWIDGET";

    var propertiesMap = {
        "action": {
            "type": "actions",
            "parent": "widget",
            "condition": "showAsAction"
        },
        "field": {
            "type": "fields",
            "parent": "widget"
        },
        "column": {
            "type": "fields",
            "parent": "widget"
        },
        "widget": {
            "type": "widgets",
            "parent": "container"
        },
        "menu": {
            "type": "menus",
            "parent": "groupMenu"
        },
        "groupMenu": {},
        "container": {
            "type": "container",
            "parent": "widgets"
        }
    };

    this.getPropertyFromScope = function(scope){
        var property = false;
        if(devToolsWindow){
            var types = Object.keys(propertiesMap);
            types.every(function(type) {
                var t =  type.charAt(0).toUpperCase() + type.slice(1);
                var instance = devToolsWindow[t];
                if(instance && (scope instanceof instance)) {
                    property = type;
                    return false;
                } else {
                    return true;
                }
            });
        }
        return property;
    };

    var getProperty = function(scope) {
        var property = CONTAINER_TYPE;
        var types = Object.keys(propertiesMap);
        types.every(function(type) {
            if(scope[type]) {
                property = type;
                return false;
            } else {
                return true;
            }
        });
        return property;
    };

    var elementHasProperties = function(element, propertiesArray){
        return propertiesArray.some(function(property){
            return element.hasOwnProperty(property);
        });
    };

    var getElementType = function(element, isMenuType){
        var elementType;
        if(isMenuType){
            elementType = 'menu';
        } else if(element[0] && getMenuType(element[0])){
            elementType = 'groupMenu';
        }
        return elementType;
    };

    var getMenuType = function(element){
        var WINDOW_NAME = 'windowName';
        var MENUS = 'menus';
        var isMenu = elementHasProperties(element, [WINDOW_NAME, MENUS]);
        var parentIsMenu = element.parent && element.parent.menus;
        var isMenuType = isMenu || parentIsMenu;
        return getElementType(element, isMenuType);
    };
    var getMetadataElement = function(scope) {
        var property = CONTAINER_TYPE;
        var completeMetadata = {
            "element": templateManager.container,
            "type": property
        };
        if (scope) {
            completeMetadata.type = self.getPropertyFromScope(scope) || getMenuType(scope);
            if(!completeMetadata.type){
                completeMetadata.type = getProperty(scope);
                completeMetadata.element = scope[completeMetadata.type];
            } else {
                completeMetadata.element = scope;
            }
        }
        return completeMetadata;
    };

    var addItemToPath = function(path, parent, parentType, item){
        var position = parent[parentType].indexOf(item);
        path.unshift(position);
        path.unshift(parentType);
        return path;
    };

    var getCompletePathToElement = function(metadataElement, type, _path) {
        var path = _path || [];
        if(type == GROUP_MENU_TYPE){
            path.splice(0, 1, GROUP_MENU_TYPE);
        } else if (type !== CONTAINER_TYPE) {
            var elementProperties = propertiesMap[type];
            var elementType = elementProperties.type;
            var parentType = elementProperties.parent;
            var parent = metadataElement.parent || metadataElement[parentType];
            if(!parent && type == MENU_TYPE){
                parent = metadataElement[PARENT_MENU] || templateManager.project;
                elementType = GROUP_MENU_TYPE;
            }
            if(parent){
                parentType = parent == metadataElement.parent ? type : parentType;
                path = addItemToPath(path, parent, elementType, metadataElement);
                getCompletePathToElement(parent, parentType, path);
            }
        }
        return path;
    };

    this.getJsonElement = function(path, container) {
        var jsonElement = container.__metadata || {};
        path.every(function(part) {
            if(!jsonElement[part]){
                jsonElement = "";
                return false;
            }
            jsonElement = jsonElement[part];
            return true;
        });
        var jsonAndPath = {
            jsonElement: Util.clone(jsonElement),
            path: path
        };
        return jsonElement ? jsonAndPath : false;
    };

    this.getPathToMetadata = function(scope){
        var pathToMetadata = [];
        var metadataElement = getMetadataElement(scope);
        var element = metadataElement.element;
        var type = metadataElement.type;
        if (element && type && type != CONTAINER_TYPE){
            pathToMetadata = getCompletePathToElement(element, type);
        }
        return pathToMetadata;
    };

    this.getElementToMetadata = function(scope, container) {
        var path = self.getPathToMetadata(scope, container);
        return self.getJsonElement(path, container);
    };
    var hasChanged = function(oldObj, newObj){
        var diffs = Object.keys(diff.getDiff(oldObj, newObj));
        var condition = ~diffs.indexOf('/fileName') ? diffs.length > 1 : diffs.length >= 1;
        return condition;
    };
    this.hasChanged = hasChanged;

    var deleteComponentInFather = function(option, index, path, somethingChange, current, key){
        var pathLength = path.length;
        var indexToCompare = option == DELETE ? pathLength - 3 : pathLength - 2;
        var currentChanged;
        if(index === indexToCompare){
            currentChanged = option == DELETE ? current[key][path[pathLength - 2]] : current.widgets;
            somethingChange = true;
            currentChanged = currentChanged.filter(function(value, index){
                return index != path[pathLength - 1];
            });
            if(option == DELETE){
                current[key][path[pathLength - 2]] = currentChanged;
            } else {
                current.widgets = currentChanged;
            }
        }
        return { current: current, somethingChange: somethingChange };
    };

    var insertNewItem = function(current, item, index){
        if(current.length <= index+1){
            current.push(item);
        } else {
            current.splice(index+1, 0, item);
        }
    };

    this.changeJson = function(json, path, completeJson, option){
        var somethingChange  = false;
        var object = json ? JSON.parse(json) : json;
        if(path && path.length > 0){
            path.reduce(function(current, key, index) {
                if(option == DELETE || option == DELETEWIDGET){
                    var changedIn = deleteComponentInFather(option, index, path, somethingChange, current, key);
                    current = changedIn.current;
                    somethingChange = changedIn.somethingChange;
                } else if(index === path.length - 1){
                    if(option == NEW){
                        insertNewItem(current, object, path[path.length - 1]);
                        somethingChange = true;
                    } else {
                        somethingChange = hasChanged(current[key], object);
                        current[key] = object;
                    }
                }
                return current[key];
            }, completeJson);
        } else {
            somethingChange = hasChanged(completeJson, object);
            completeJson = object;
        }
        return { somethingChange: somethingChange, completeJson : completeJson };
    };
}